"""Alpine Linux logo"""

ALPINE_LINUX = [
    """{c[0]}        .hddddddddddddddddddddddh.          """,
    """{c[0]}       :dddddddddddddddddddddddddd:         """,
    """{c[0]}      /dddddddddddddddddddddddddddd/        """,
    """{c[0]}     +dddddddddddddddddddddddddddddd+       """,
    """{c[0]}   `sdddddddddddddddddddddddddddddddds`     """,
    """{c[0]}  `ydddddddddddd++hdddddddddddddddddddy`    """,
    """{c[0]} .hddddddddddd+`  `+ddddh:-sdddddddddddh.   """,
    """{c[0]} hdddddddddd+`      `+y:    .sddddddddddh   """,
    """{c[0]} ddddddddh+`   `//`   `.`     -sddddddddd   """,
    """{c[0]} ddddddh+`   `/hddh/`   `:s-    -sddddddd   """,
    """{c[0]} ddddh+`   `/+/dddddh/`   `+s-    -sddddd   """,
    """{c[0]} ddd+`   `/o` :dddddddh/`   `oy-    .yddd   """,
    """{c[0]} hdddyo+ohddyosdddddddddho+oydddy++ohdddh   """,
    """{c[0]} .hddddddddddddddddddddddddddddddddddddh.   """,
    """{c[0]}  `yddddddddddddddddddddddddddddddddddy`    """,
    """{c[0]}   `sdddddddddddddddddddddddddddddddds`     """,
    """{c[0]}     +dddddddddddddddddddddddddddddd+       """,
    """{c[0]}      /dddddddddddddddddddddddddddd/        """,
    """{c[0]}       :dddddddddddddddddddddddddd:         """,
    """{c[0]}        .hddddddddddddddddddddddh.          """
]
